'use strict';

const Bech32 = {
    encode: encode$2,
    decode: decode$2
};
const CHARSET = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const encodings = {
    BECH32: 'bech32',
    BECH32M: 'bech32m'
};
function getEncodingConst(enc) {
    switch (enc) {
        case encodings.BECH32:
            return 1;
        case encodings.BECH32M:
            return 0x2bc830a3;
        default:
            throw new Error(`Unrecognized encoding: ${enc}`);
    }
}
function polymod(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if ((top >> i) & 1) {
                chk ^= GENERATOR[i];
            }
        }
    }
    return chk;
}
function hrpExpand(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function convertBits(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits)) {
            return [];
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || (acc << (toBits - bits)) & maxVal) {
        return [];
    }
    return ret;
}
function verifyChecksum(hrp, data, enc) {
    const combined = hrpExpand(hrp).concat(data);
    return polymod(combined) === getEncodingConst(enc);
}
function createChecksum(hrp, data, enc) {
    const values = hrpExpand(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod(values) ^ getEncodingConst(enc);
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function b32encode(hrp, data, enc) {
    const combined = data.concat(createChecksum(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET.charAt(combined[p]);
    }
    return ret;
}
function b32decode(bechstr, version) {
    const enc = (version) ? 'bech32m' : 'bech32';
    if (!checkBounds(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos(bechstr)) {
        return { hrp: null, data: [0xFF] };
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET.indexOf(bechstr.charAt(p));
        if (d === -1) {
            return { hrp: null, data: [0xFF] };
        }
        data.push(d);
    }
    return (verifyChecksum(hrp, data, enc))
        ? { hrp, data: data.slice(0, data.length - 6) }
        : { hrp: null, data: [0xFF] };
}
function checkBounds(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    return !(hasLower && hasUpper);
}
function checkSeparatorPos(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function encode$2(data, hrp = 'bch', version = 0) {
    const dat = [version, ...convertBits([...data], 8, 5)];
    const enc = (version) ? 'bech32m' : 'bech32';
    const str = b32encode(hrp, dat, enc);
    const chk = decode$2(str, version);
    if (!chk) {
        throw new Error(`Failed to decode string: ${str}`);
    }
    return str;
}
function decode$2(str, version = 0) {
    const hrp = str.split('1', 1)[0];
    const { hrp: hrpgot, data } = b32decode(str, version);
    const decoded = convertBits(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        case (data[0] === 0 && length !== 20 && length !== 32):
            throw new Error('Decoded string does not match version 0 spec.');
        case (data[0] === 0 && version !== 0):
            throw new Error('Decoded version bit does not match.');
        case (data[0] !== 0 && version !== 1):
            throw new Error('Decoded version bit does not match.');
        default:
            return Uint8Array.from(decoded);
    }
}
var Bech32$1 = Bech32;

const ALPHABET = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz';
const Base58 = {
    encode: encode$1,
    decode: decode$1
};
function encode$1(bytes) {
    const d = [];
    let s = '';
    let i, j = 0, c, n;
    for (i of bytes) {
        j = 0;
        c = bytes[i];
        s += c || s.length ^ i ? '' : 1;
        while (j in d || c) {
            n = d[j];
            n = n ? n * 256 + c : c;
            c = n / 58 | 0;
            d[j] = n % 58;
            j++;
        }
    }
    while (j--) {
        s += ALPHABET[d[j]];
    }
    return s;
}
function decode$1(str) {
    const d = [], b = [];
    let i, j = 0, c, n;
    for (i = 0; i < str.length; i++) {
        j = 0;
        c = ALPHABET.indexOf(str[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        i = (c || b.length ^ i) ? i : b.push(0);
        while (j in d || c) {
            n = d[j];
            n = n ? n * 58 + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j--) {
        b.push(d[j]);
    }
    return Uint8Array.from(b);
}
var Base58$1 = Base58;

const Base64 = {
    encode,
    decode
};
function encode(str) {
    if (typeof window !== 'undefined') {
        return btoa(str);
    }
    return Buffer.from(str).toString('base64');
}
function decode(str) {
    if (typeof window !== 'undefined') {
        return atob(str);
    }
    return Buffer.from(str, 'base64').toString('utf8');
}
var Base64$1 = Base64;

class Buff extends Uint8Array {
    constructor(data, size) {
        if (size) {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        this.toArr = () => Array.from(this);
        this.toStr = () => bytesToStr(this);
        this.toNum = () => bytesToNum(this);
        this.toBig = () => bytesToBig(this);
        this.toHex = () => bytesToHex(this);
        this.toJson = () => JSON.parse(bytesToStr(this));
        this.toBytes = () => new Uint8Array(this);
        this.toBase32 = () => Bech32$1.encode(this);
        this.toBase58 = () => Base58$1.encode(this);
        this.toBase64 = () => Base64$1.encode(this);
        return this;
    }
}
Buff.str = (x, s) => new Buff(strToBytes(x), s);
Buff.hex = (x, s) => new Buff(hexToBytes(x), s);
Buff.num = (x, s) => new Buff(numToBytes(x), s);
Buff.big = (x, s) => new Buff(bigToBytes(x), s);
Buff.buff = (x, s) => new Buff(x, s);
Buff.json = (x) => new Buff(strToBytes(JSON.stringify(x)));
Buff.bech32 = (x) => new Buff(Bech32$1.decode(x));
Buff.base58 = (x) => new Buff(Base58$1.decode(x));
Buff.base64 = (x) => new Buff(Base64$1.decode(x));
function strToBytes(str) {
    const ec = new TextEncoder();
    return ec.encode(str).buffer;
}
function hexToBytes(str) {
    const bytes = [];
    let i, idx = 0;
    if (str.length % 2) {
        throw new Error(`Invalid hex string length: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return Uint8Array.from(bytes).buffer;
}
function numToBytes(num) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return Uint8Array.from(bytes).buffer;
}
function bigToBytes(big) {
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return Uint8Array.from(bytes).buffer;
}
function bytesToStr(bytes) {
    const dc = new TextDecoder();
    return dc.decode(bytes);
}
function bytesToHex(bytes) {
    const hex = [];
    let i;
    for (i = 0; i < bytes.length; i++) {
        hex.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return hex.join('');
}
function bytesToNum(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}

class Bytes extends Buff {
    constructor(data, size) {
        super(data, size);
        this.size = this.length;
    }
    prepend(data) {
        return Bytes.of(...data, ...this);
    }
    append(data) {
        return Bytes.of(...this, ...data);
    }
    varint(num) {
        return Bytes.of(...this, ...Bytes.varint(num));
    }
    static from(data) {
        return new Bytes(Uint8Array.from(data).buffer);
    }
    static of(...args) {
        return new Bytes(Uint8Array.of(...args).buffer);
    }
    static join(arr) {
        let i, idx = 0;
        const totalSize = arr.reduce((prev, curr) => prev + curr.length, 0);
        const totalBytes = new Uint8Array(totalSize);
        for (const bytes of arr) {
            for (i = 0; i < bytes.length; idx++, i++) {
                totalBytes[idx] = bytes[i];
            }
        }
        return new Bytes(totalBytes, totalSize);
    }
    static varint(num) {
        if (num < 0xFD) {
            return Buff.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff.of(0xFD, ...Buff.num(num, 2));
        }
        else if (num < 0x100000000) {
            return Buff.of(0xFE, ...Buff.num(num, 4));
        }
        else if (num < 0x10000000000000000) {
            return Buff.of(0xFF, ...Buff.num(num, 8));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
}

class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size, reverse = false) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        const chunk = (reverse)
            ? this.data.slice(0, size).reverse()
            : this.data.slice(0, size);
        return Bytes.from(chunk);
    }
    read(size, reverse = false) {
        size = size || this.varint();
        const chunk = this.peek(size, reverse);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    varint() {
        const num = this.read(1).toNum();
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum();
            case (num === 0xFE):
                return this.read(4).toNum();
            case (num === 0xFF):
                return this.read(8).toNum();
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
}

class Type {
    static of(x) {
        for (const [k, v] of Object.entries(Type.is)) {
            if (v(x)) {
                return k;
            }
        }
        return 'unknown';
    }
}
Type.is = {
    undefined: { k: 'undefined', v: x => typeof x === 'undefined' },
    infinity: { k: 'infinity', v: x => x === Infinity },
    null: { k: 'null', v: x => x === null },
    hex: { k: 'hex', v: x => isHex(x) },
    string: { k: 'string', v: x => typeof x === 'string' },
    bigint: { k: 'bigint', v: x => typeof x === 'bigint' },
    number: { k: 'number', v: x => typeof x === 'number' },
    array: { k: 'array', v: x => Array.isArray(x) },
    uint8: { k: 'uint8', v: x => x instanceof Uint8Array },
    uint16: { k: 'uint16', v: x => x instanceof Uint16Array },
    uint32: { k: 'uint32', v: x => x instanceof Uint32Array },
    buffer: { k: 'buffer', v: x => x instanceof ArrayBuffer },
    object: { k: 'object', v: x => typeof x === 'object' }
};
Type.array = {
    isString: x => x.every((e) => Type.is.string.v(e)),
    isNumber: x => x.every((e) => Type.is.number.v(e)),
    isBigint: x => x.every((e) => Type.is.bigint.v(e))
};
function isHex(str) {
    switch (true) {
        case (typeof str !== 'string'):
            return false;
        case (/[^0-9a-fA-F]/.test(str)):
            return false;
        case (str.length % 2 !== 0):
            return false;
        default:
            return true;
    }
}

exports.Buff = Buff;
exports.Bytes = Bytes;
exports.Stream = Stream;
exports.Type = Type;
//# sourceMappingURL=main.js.map
